import React, { useState, useEffect } from 'react';
import { createClient } from '@supabase/supabase-js';
import { Wand2, Sparkles, Image as ImageIcon, Download, Share2, Coins, LogIn, LogOut, Settings, AlertCircle, Zap, Palette } from 'lucide-react';
import { AuthModal } from './components/AuthModal';
import { SubscriptionModal } from './components/SubscriptionModal';
import { CoinPurchaseModal } from './components/CoinPurchaseModal';
import OpenAI from 'openai';

// Initialize Supabase client
const supabase = createClient(
  import.meta.env.VITE_SUPABASE_URL,
  import.meta.env.VITE_SUPABASE_ANON_KEY,
  {
    auth: {
      persistSession: true,
      detectSessionInUrl: true,
      autoRefreshToken: true,
    }
  }
);

// Initialize OpenAI client
const openai = new OpenAI({
  apiKey: 'dummy-key', // Will be replaced with server-side API key
  dangerouslyAllowBrowser: true // For demo purposes only
});

// API endpoints for different models
const API_BASE_URL = 'https://ar-api-08uk.onrender.com/arimage/';

function App() {
  // State for user authentication
  const [user, setUser] = useState<any>(null);
  const [loading, setLoading] = useState(true);
  const [showAuthModal, setShowAuthModal] = useState(false);
  
  // State for image generation
  const [prompt, setPrompt] = useState('');
  const [generatedImage, setGeneratedImage] = useState<string | null>(null);
  const [generating, setGenerating] = useState(false);
  const [error, setError] = useState<string | null>(null);
  
  // State for model selection
  const [selectedModel, setSelectedModel] = useState<'standard' | 'turbo'>('standard');
  const [selectedApiModel, setSelectedApiModel] = useState<'dechore' | 'flux' | 'turbo'>('dechore');
  const [hasSubscription, setHasSubscription] = useState(false);
  const [showSubscriptionModal, setShowSubscriptionModal] = useState(false);
  
  // State for coin system
  const [coins, setCoins] = useState(0);
  const [showCoinPurchaseModal, setShowCoinPurchaseModal] = useState(false);
  
  // Check for payment status in URL
  useEffect(() => {
    const checkPaymentStatus = () => {
      const urlParams = new URLSearchParams(window.location.search);
      const status = urlParams.get('status');
      const txnid = urlParams.get('txnid');
      const coins = urlParams.get('coins');
      
      if (status === 'success' && txnid && coins) {
        const coinAmount = parseInt(coins);
        if (!isNaN(coinAmount)) {
          // Update coins in UI immediately
          setCoins(prevCoins => prevCoins + coinAmount);
          
          // Show success message
          setError(`Payment successful! ${coinAmount} coins added to your account.`);
          setTimeout(() => setError(null), 5000);
        }
        
        // Clear URL parameters
        window.history.replaceState({}, '', window.location.pathname);
      } else if (status === 'failure' && txnid) {
        setError('Payment failed. Please try again or contact support.');
        
        // Clear URL parameters
        window.history.replaceState({}, '', window.location.pathname);
      }
    };
    
    checkPaymentStatus();
  }, []);
  
  // Check authentication status on mount
  useEffect(() => {
    const checkUser = async () => {
      try {
        const { data: { user } } = await supabase.auth.getUser();
        setUser(user);
        
        if (user) {
          // Check subscription status
          const { data: subscriptions } = await supabase
            .from('subscriptions')
            .select('*')
            .eq('user_id', user.id)
            .eq('status', 'active')
            .limit(1);
          
          const hasActiveSubscription = subscriptions && subscriptions.length > 0;
          setHasSubscription(hasActiveSubscription);
          
          if (hasActiveSubscription) {
            setSelectedModel('turbo');
          }
          
          // Get user coins
          const { data: userCoins } = await supabase
            .from('user_coins')
            .select('coins')
            .eq('user_id', user.id)
            .limit(1);
          
          if (userCoins && userCoins.length > 0) {
            setCoins(userCoins[0].coins);
          }
        }
      } catch (error) {
        console.error('Error checking auth status:', error);
      } finally {
        setLoading(false);
      }
    };
    
    checkUser();
    
    // Set up auth state change listener
    const { data: { subscription } } = supabase.auth.onAuthStateChange(
      async (event, session) => {
        if (event === 'SIGNED_IN' && session?.user) {
          setUser(session.user);
          
          // Check subscription status
          const { data: subscriptions } = await supabase
            .from('subscriptions')
            .select('*')
            .eq('user_id', session.user.id)
            .eq('status', 'active')
            .limit(1);
          
          const hasActiveSubscription = subscriptions && subscriptions.length > 0;
          setHasSubscription(hasActiveSubscription);
          
          if (hasActiveSubscription) {
            setSelectedModel('turbo');
          }
          
          // Get user coins
          const { data: userCoins } = await supabase
            .from('user_coins')
            .select('coins')
            .eq('user_id', session.user.id)
            .limit(1);
          
          if (userCoins && userCoins.length > 0) {
            setCoins(userCoins[0].coins);
          }
        } else if (event === 'SIGNED_OUT') {
          setUser(null);
          setHasSubscription(false);
          setSelectedModel('standard');
          setCoins(0);
        }
      }
    );
    
    return () => {
      subscription.unsubscribe();
    };
  }, []);
  
  const handleLogin = () => {
    setShowAuthModal(true);
  };
  
  const handleLogout = async () => {
    await supabase.auth.signOut();
    setUser(null);
    setHasSubscription(false);
    setSelectedModel('standard');
    setCoins(0);
  };
  
  const handleAuthSuccess = () => {
    // Auth successful, modal will close automatically
  };
  
  const handleSubscriptionSuccess = () => {
    setHasSubscription(true);
    setSelectedModel('turbo');
  };
  
  const handleCoinPurchaseSuccess = (amount: number) => {
    setCoins(prevCoins => prevCoins + amount);
  };
  
  const handleGenerateImage = async () => {
    if (!prompt.trim()) {
      setError('Please enter a prompt');
      return;
    }
    
    setError(null);
    setGenerating(true);
    
    try {
      // Check if user has enough coins for artistic mode
      if (selectedApiModel === 'turbo') {
        if (!user) {
          setError('Please sign in to use Artistic mode');
          setGenerating(false);
          return;
        }
        
        if (!hasSubscription && coins < 1) {
          setError('Not enough coins. Please purchase coins or subscribe to use Artistic mode');
          setGenerating(false);
          return;
        }
        
        // Deduct coins if not subscribed
        if (!hasSubscription) {
          const newCoins = coins - 1;
          setCoins(newCoins);
          
          // Update coins in database
          if (user) {
            await supabase
              .from('user_coins')
              .upsert([
                { user_id: user.id, coins: newCoins }
              ]);
            
            // Record transaction
            await supabase
              .from('coin_transactions')
              .insert([
                {
                  user_id: user.id,
                  amount: -1,
                  transaction_type: 'usage',
                  description: 'Generated image in Artistic mode'
                }
              ]);
          }
        }
      }
      
      // Call the API with the selected model
      const apiUrl = `${API_BASE_URL}?prompt=${encodeURIComponent(prompt.trim())}&model=${selectedApiModel}`;
      
      try {
        const response = await fetch(apiUrl);
        
        if (!response.ok) {
          throw new Error(`API error: ${response.status}`);
        }
        
        const imageBlob = await response.blob();
        const imageUrl = URL.createObjectURL(imageBlob);
        setGeneratedImage(imageUrl);
      } catch (apiError) {
        console.error('API error:', apiError);
        
        // Fallback to placeholder image if API fails
        const imageKeywords = encodeURIComponent(prompt.trim());
        const placeholderImage = `https://source.unsplash.com/random/512x512/?${imageKeywords}`;
        setGeneratedImage(placeholderImage);
      }
      
      // Save image to database
      if (user) {
        try {
          await supabase
            .from('images')
            .insert([
              {
                url: generatedImage || 'placeholder',
                prompt: prompt.trim(),
                user_id: user.id,
                model: selectedApiModel
              }
            ]);
        } catch (err) {
          console.error('Error saving image to database:', err);
          // Continue even if database save fails
        }
      }
    } catch (err: any) {
      console.error('Error generating image:', err);
      setError(err.message || 'Failed to generate image');
    } finally {
      setGenerating(false);
    }
  };
  
  const handleDownload = () => {
    if (!generatedImage) return;
    
    const link = document.createElement('a');
    link.href = generatedImage;
    link.download = `ai-image-${Date.now()}.jpg`;
    document.body.appendChild(link);
    link.click();
    document.body.removeChild(link);
  };
  
  const handleShare = async () => {
    if (!generatedImage) return;
    
    try {
      if (navigator.share) {
        await navigator.share({
          title: 'My AI Generated Image',
          text: `Check out this AI image I created with the prompt: "${prompt}"`,
          url: generatedImage
        });
      } else {
        // Fallback for browsers that don't support Web Share API
        navigator.clipboard.writeText(generatedImage);
        setError('Image URL copied to clipboard!');
        setTimeout(() => setError(null), 3000);
      }
    } catch (err) {
      console.error('Error sharing:', err);
    }
  };
  
  return (
    <div className="animated-bg min-h-screen">
      <div className="container mx-auto px-4 py-8">
        {/* Header */}
        <header className="flex justify-between items-center mb-8">
          <div className="flex items-center">
            <Wand2 className="w-8 h-8 text-blue-400 mr-2" />
            <h1 className="text-2xl font-bold text-white">Free AI Image Generator</h1>
          </div>
          
          <div className="flex items-center gap-4">
            {user ? (
              <>
                <div className="flex items-center bg-gray-800/50 rounded-full px-3 py-1">
                  <Coins className="w-4 h-4 text-yellow-400 mr-1" />
                  <span className="text-white text-sm font-medium">{coins}</span>
                  <button 
                    onClick={() => setShowCoinPurchaseModal(true)}
                    className="ml-2 text-xs text-blue-400 hover:text-blue-300"
                  >
                    + Add
                  </button>
                </div>
                
                <div className="relative group">
                  <button className="w-10 h-10 rounded-full bg-gray-700 flex items-center justify-center text-white">
                    {user.email?.charAt(0).toUpperCase() || <Settings className="w-5 h-5" />}
                  </button>
                  
                  <div className="absolute right-0 mt-2 w-48 bg-gray-800 rounded-lg shadow-lg py-2 z-10 hidden group-hover:block">
                    <div className="px-4 py-2 border-b border-gray-700">
                      <p className="text-sm text-gray-300 truncate">{user.email}</p>
                    </div>
                    
                    <button
                      onClick={handleLogout}
                      className="w-full text-left px-4 py-2 text-sm text-gray-300 hover:bg-gray-700 flex items-center"
                    >
                      <LogOut className="w-4 h-4 mr-2" />
                      Sign Out
                    </button>
                  </div>
                </div>
              </>
            ) : (
              <button
                onClick={handleLogin}
                className="flex items-center gap-2 bg-blue-600 hover:bg-blue-700 text-white px-4 py-2 rounded-lg transition-colors"
              >
                <LogIn className="w-4 h-4" />
                Sign In
              </button>
            )}
          </div>
        </header>
        
        {/* Main content */}
        <main className="max-w-3xl mx-auto">
          <div className="bg-gray-900/60 backdrop-blur-sm rounded-xl p-6 shadow-xl mb-8">
            <h2 className="text-xl font-semibold text-white mb-4">Create Your Image</h2>
            
            <div className="space-y-6">
              {/* Prompt input */}
              <div>
                <label htmlFor="prompt" className="block text-sm font-medium text-gray-300 mb-2">
                  Describe the image you want to create
                </label>
                <textarea
                  id="prompt"
                  rows={3}
                  value={prompt}
                  onChange={(e) => setPrompt(e.target.value)}
                  placeholder="A futuristic city with flying cars and neon lights..."
                  className="w-full px-4 py-2 bg-gray-800 border border-gray-700 rounded-lg text-white placeholder-gray-500 focus:outline-none focus:ring-2 focus:ring-blue-500"
                />
              </div>
              
              {/* Generate button */}
              <button
                onClick={handleGenerateImage}
                disabled={generating || !prompt.trim()}
                className="w-full py-3 bg-gradient-to-r from-blue-600 to-purple-600 text-white rounded-lg font-medium hover:from-blue-700 hover:to-purple-700 transition-all disabled:opacity-50 disabled:cursor-not-allowed flex items-center justify-center gap-2"
              >
                {generating ? (
                  <>
                    <div className="w-5 h-5 rounded-full border-2 border-white border-t-transparent animate-spin" />
                    Generating...
                  </>
                ) : (
                  <>
                    <Wand2 className="w-5 h-5" />
                    Generate Image
                  </>
                )}
              </button>
              
              {/* Model selection */}
              <div>
                <label className="block text-sm font-medium text-gray-300 mb-2">
                  Select Model (Optional)
                </label>
                <div className="grid grid-cols-3 gap-3">
                  <button
                    onClick={() => setSelectedApiModel('dechore')}
                    className={`p-3 rounded-lg border flex flex-col items-center ${
                      selectedApiModel === 'dechore'
                        ? 'border-green-500 bg-green-900/30'
                        : 'border-gray-700 bg-gray-800/50 hover:bg-gray-800'
                    }`}
                  >
                    <Zap className="w-5 h-5 text-green-400 mb-1" />
                    <span className="font-medium text-white">Fast</span>
                    <span className="text-xs text-gray-400 mt-1">Decohere AI</span>
                  </button>
                  
                  <button
                    onClick={() => setSelectedApiModel('flux')}
                    className={`p-3 rounded-lg border flex flex-col items-center ${
                      selectedApiModel === 'flux'
                        ? 'border-blue-500 bg-blue-900/30'
                        : 'border-gray-700 bg-gray-800/50 hover:bg-gray-800'
                    }`}
                  >
                    <ImageIcon className="w-5 h-5 text-blue-400 mb-1" />
                    <span className="font-medium text-white">Standard</span>
                    <span className="text-xs text-gray-400 mt-1">FLUX AI</span>
                  </button>
                  
                  <button
                    onClick={() => {
                      if (!user) {
                        handleLogin();
                        return;
                      }
                      
                      if (!hasSubscription && coins < 1) {
                        setShowCoinPurchaseModal(true);
                        return;
                      }
                      
                      setSelectedApiModel('turbo');
                    }}
                    className={`p-3 rounded-lg border flex flex-col items-center ${
                      selectedApiModel === 'turbo'
                        ? 'border-purple-500 bg-purple-900/30'
                        : 'border-gray-700 bg-gray-800/50 hover:bg-gray-800'
                    }`}
                  >
                    <Palette className="w-5 h-5 text-purple-400 mb-1" />
                    <span className="font-medium text-white">Artistic</span>
                    {hasSubscription ? (
                      <span className="text-xs text-green-400 mt-1">Unlimited</span>
                    ) : (
                      <span className="text-xs text-gray-400 mt-1">1 coin/image</span>
                    )}
                  </button>
                </div>
                
                {!hasSubscription && selectedApiModel === 'turbo' && (
                  <div className="mt-2 flex justify-between items-center">
                    <span className="text-xs text-gray-400">
                      {coins >= 1 
                        ? `You have ${coins} coins (${coins} images)`
                        : 'Not enough coins'}
                    </span>
                    <button
                      onClick={() => setShowSubscriptionModal(true)}
                      className="text-xs text-purple-400 hover:text-purple-300"
                    >
                      Get unlimited access
                    </button>
                  </div>
                )}
              </div>
              
              {/* Error message */}
              {error && (
                <div className={`flex items-center gap-2 p-3 rounded-lg ${
                  error.includes('successful') 
                    ? 'bg-green-900/20 text-green-400' 
                    : 'bg-red-900/20 text-red-400'
                }`}>
                  <AlertCircle className="w-5 h-5 flex-shrink-0" />
                  <p className="text-sm">{error}</p>
                </div>
              )}
            </div>
          </div>
          
          {/* Generated image */}
          {generatedImage && (
            <div className="bg-gray-900/60 backdrop-blur-sm rounded-xl p-6 shadow-xl">
              <h2 className="text-xl font-semibold text-white mb-4">Your Generated Image</h2>
              
              <div className="rounded-lg overflow-hidden mb-4">
                <img 
                  src={generatedImage} 
                  alt={prompt} 
                  className="w-full h-auto object-cover"
                  loading="lazy"
                />
              </div>
              
              <div className="flex gap-3">
                <button
                  onClick={handleDownload}
                  className="flex-1 py-2 bg-gray-800 hover:bg-gray-700 text-white rounded-lg font-medium transition-colors flex items-center justify-center gap-2"
                >
                  <Download className="w-5 h-5" />
                  Download
                </button>
                
                <button
                  onClick={handleShare}
                  className="flex-1 py-2 bg-gray-800 hover:bg-gray-700 text-white rounded-lg font-medium transition-colors flex items-center justify-center gap-2"
                >
                  <Share2 className="w-5 h-5" />
                  Share
                </button>
              </div>
            </div>
          )}
        </main>
        
        {/* Footer */}
        <footer className="mt-12 text-center text-gray-400 text-sm">
          <p>© {new Date().getFullYear()} Free AI Image Generator. All rights reserved.</p>
          <p className="mt-1">
            <a href="#" className="hover:text-white">Privacy Policy</a>
            {' • '}
            <a href="#" className="hover:text-white">Terms of Service</a>
          </p>
        </footer>
      </div>
      
      {/* Modals */}
      <AuthModal
        isOpen={showAuthModal}
        onClose={() => setShowAuthModal(false)}
        onSuccess={handleAuthSuccess}
      />
      
      <SubscriptionModal
        isOpen={showSubscriptionModal}
        onClose={() => setShowSubscriptionModal(false)}
        onSuccess={handleSubscriptionSuccess}
      />
      
      <CoinPurchaseModal
        isOpen={showCoinPurchaseModal}
        onClose={() => setShowCoinPurchaseModal(false)}
        onSuccess={handleCoinPurchaseSuccess}
      />
    </div>
  );
}

export default App;